@@ -1,6 +1,6 @@
 #######################################################################
 # stream -  Infrastructure for Data Stream Mining
-# Copyright (C) 2013 Michael Hahsler, Matthew Bolanos, John Forrest 
+# Copyright (C) 2013 Michael Hahsler, Matthew Bolanos, John Forrest
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,18 +20,26 @@
 DSD_UniformNoise <- function(d=2, range=NULL) {
   if(is.null(range)) range <- matrix(c(0,1), ncol=2, nrow=d, byrow=TRUE)
   structure(list(description = "Uniform Noise", d = d, range=range),
-	      class=c("DSD_UniformNoise", "DSD_R", "DSD_data.frame", "DSD"))
-  }
-  
-get_points.DSD_UniformNoise <- function(x, n=1, 
-    cluster=FALSE, class=FALSE, ...) {
+    class=c("DSD_UniformNoise", "DSD_R", "DSD_data.frame", "DSD"))
+}
+
+get_points.DSD_UniformNoise <- function(x, n=1,
+  outofpoints=c("stop", "warn", "ignore"),
+  cluster=FALSE, class=FALSE, ...) {
   .nodots(...)
 
-    data <- as.data.frame(t(replicate(n, 
-      runif(x$d, min=x$range[,1], max=x$range[,2]))))
-    
-    if(cluster) attr(data, "cluster") <- rep(NA_integer_, n)
-    if(class) data <- cbind(data, class = rep(NA_integer_, n))
-    
-    data
+  # outofpoints is ignored
+
+  data <- t(replicate(n,
+    runif(x$d, min=x$range[,1], max=x$range[,2])))
+
+  ### handle 1-d case
+  if(x$d == 1) data <- t(data)
+
+  data <- data.frame(data)
+
+  if(cluster) attr(data, "cluster") <- rep(NA_integer_, n)
+  if(class) data <- cbind(data, class = rep(NA_integer_, n))
+
+  data
 }
