@@ -9,4 +9,6 @@
 ## For R 2.15.1 and later this also works. Note that calling loadModule() triggers
 ## a load action, so this does not have to be placed in .onLoad() or evalqOnLoad().
 loadModule("MOD_DBSTREAM", TRUE)
-loadModule("MOD_DStream", TRUE)
\ No newline at end of file
+loadModule("MOD_DStream", TRUE)
+loadModule("MOD_BICO", TRUE)
+loadModule("MOD_BIRCH", TRUE)
\ No newline at end of file
